"""
Get all stock name
"""

import requests
from io import StringIO
import pandas as pd
import os
import shutil
import sys

ALL_STOCKS_URL = 'http://218.244.146.57/static/all.csv'


def get_cache_dir():
    home_path = os.path.expanduser('~')
    cache_dir = os.path.join(home_path, '.zipline_cn_databundle')

    if not os.path.isdir(cache_dir):
        os.makedirs(cache_dir)
    return cache_dir

def get_cache_path():
    cache_dir = get_cache_dir()
    return os.path.join(cache_dir, 'all_stocks.csv')

def get_all_stocks(cache=True):
    """
    used idea from tushare
    :return: data frame of stock list
    """
    cache_path = get_cache_path()

    #use cache
    if cache and os.path.isfile(cache_path):
        df = pd.read_csv(cache_path,  dtype={'code': 'object'})
        df = df.set_index('code')
        return df

    response = requests.get(ALL_STOCKS_URL)
    text = response.content
    text = text.decode('GBK')
    text = text.replace('--', '')
    df = pd.read_csv(StringIO(text), dtype={'code': 'object'})
    df = df.set_index('code')
    # write to cache
    with open(cache_path, 'wb') as f:
        f.write(text.encode('utf-8'))

    return df



if __name__ == '__main__':
    print(get_all_stocks(True))


import click
import tushare as ts
import pandas as pd
import os
from .squant_source import load_splits_and_dividends, zipline_splits_and_dividends

"""
从tushare获取股票信息

其中....

需要读取通联数据, 请设置 环境变量

ZIPLINE_TL_TOKEN

保存通联数据的token

"""

"""
ZIPLINE_TL_TOKEN = os.environ.get('ZIPLINE_TL_TOKEN')

if not ZIPLINE_TL_TOKEN:
    raise Exception("no datayes token in envirionment ZIPLINE_TL_TOKEN,  we need this token to fetch ajustments data")

ts.set_token('ZIPLINE_TL_TOKEN')
"""

def tushare_bundle(environ,
                  asset_db_writer,
                  minute_bar_writer,
                  daily_bar_writer,
                  adjustment_writer,
                  calendar,
                  start_session,
                  end_session,
                  cache,
                  show_progress,
                  output_dir):

    metadata, histories, symbol_map = get_basic_info()
    # 写入股票基础信息
    asset_db_writer.write(metadata)
    # 准备写入dailybar
    daily_bar_writer.write(get_hist_data(symbol_map, histories, start_session, end_session), show_progress=show_progress)
    # 送股,分红数据, 从squant 获取
    splits, dividends = zipline_splits_and_dividends(symbol_map)
    adjustment_writer.write(
        splits=pd.concat(splits, ignore_index=True),
        dividends=pd.concat(dividends, ignore_index=True),
    )

def get_basic_info(show_progress=True):
    # 先获取列表
    if show_progress:
        click.echo("获取股票基础信息")
    ts_symbols = ts.get_stock_basics()
    if show_progress:
        click.echo("写入股票列表")

    symbols = []

    histories = {}

    # 获取股票数据
    i = 0
    total = len(ts_symbols)
    for index, row in ts_symbols.iterrows():
        i = i +1
        if i > 10:
            break

        srow = {}
        # 获取历史报价信息
        click.echo("正在获取代码%s(%s)的历史行情信息 (%d/%d)" % (index, row['name'], i, total))
        histories[index] = ts.get_hist_data(index)
        srow['start_date'] = histories[index].index[-1]
        srow['end_date'] = histories[index].index[0]
        srow['symbol'] = index
        srow['asset_name'] = row['name']
        symbols.append(srow)

    df_symbols = pd.DataFrame(data=symbols).sort_values('symbol')
    symbol_map = pd.DataFrame.copy(df_symbols.symbol)

    # fix the symbol exchange info
    df = df_symbols.apply(func=convert_symbol_series, axis=1)


    return df, histories, symbol_map


def symbol_to_exchange(symbol):
    isymbol = int(symbol)
    if (isymbol>=600000):
        return symbol + ".SS", "SSE"
    else:
        return symbol + ".SZ", "SZSE"

def convert_symbol_series(s):
    symbol, e = symbol_to_exchange(s['symbol'])
    s['symbol'] = symbol
    s['exchange'] = e
    return s

def get_hist_data(symbol_map, histories, start_session, end_session):
    for sid, index in symbol_map.iteritems():
        history = histories[index]

        """
        writer needs format with
        [index], open, close, high, low, volume

        so we do not need to change the format from tushare

        but we need resort it
        """
        yield sid, history.sort_index()

if __name__ == '__main__':
    df_symbols, histories, symbol_map = get_basic_info()
    print(df_symbols)

    """
    for h,df in histories.items():
        print(df)
    """



from squant.data.stock import file_parser
from squant.zipline.datasource import get_symbol_list
import os
import datetime
from .tdx.reader import TdxReader, TdxFileNotFoundException
import pandas as pd

"""
Squant is a private library that parse the data from our private source

That read data from binary files

via RainX<i@rainx.cn>

ipython 3 only
"""


# 需要设定沪深文件目录
CQCX_SH = os.environ.get("CQCX_SH")
CQCX_SZ = os.environ.get("CQCX_SZ")

TDX_DIR = os.environ.get("TDX_DIR")

if not CQCX_SH or not CQCX_SZ:
    raise Exception("need set cqcx file on CQCX_SH CQCX_SZ")

if not os.path.isfile(CQCX_SH) \
    or not os.path.isfile(CQCX_SZ):
    raise Exception("setting CQCX_SH, CQCX_SZ path is not correct")


if not TDX_DIR:
    raise Exception("Please Setting TDX data dir")

CQCX_LIST = (CQCX_SH, CQCX_SZ)

def load_splits_and_dividends():
    """
    获取所有除权出息的信息, 根据zipline平台的特点,忽略配股信息
    :return:
    """

    splits = {}
    dividends = {}

    for CQCX in CQCX_LIST:
        cqcx_data = file_parser.get_cqcx(CQCX.encode("utf-8"))
        for row in cqcx_data:
            code = str(row['stock']).zfill(6)
            # sgVal 送股数，每1000股送股数
            if row['sgVal'] != 0:
                if code not in splits.keys():
                    splits[code] = []
                splits[code].append({
                    'effective_date' : int_to_date(row['date']),
                    'ratio' : 1000 / (1000 + row['sgVal']),
                })

            if row['pxVal'] != 0:
                if code not in dividends.keys():
                    dividends[code] = []
                dividends[code].append({
                    'amount' : row['pxVal'] / 1000,
                    'ex_date' : int_to_date(row['date']),
                })


    return splits, dividends


def zipline_splits_and_dividends(symbol_map):
    raw_splits, raw_dividends = load_splits_and_dividends()
    splits = []
    dividends = []
    for sid, code in symbol_map.iteritems():
        if code in raw_splits:
            split = pd.DataFrame(data=raw_splits[code])
            split['sid'] = sid
            split.index = split['effective_date'] = pd.DatetimeIndex(split['effective_date'])
            splits.append(split)
        if code in raw_dividends:
            dividend = pd.DataFrame(data = raw_dividends[code])
            dividend['sid'] = sid
            dividend['record_date'] = dividend['declared_date'] = dividend['pay_date'] = pd.NaT
            dividend.index = dividend['ex_date'] = pd.DatetimeIndex(dividend['ex_date'])
            dividends.append(dividend)
    return splits, dividends

def int_to_date(d):
    d = str(d)
    return datetime.date(int(d[:4]), int(d[4:6]), int(d[6:]))


def squant_bundle(environ,
                   asset_db_writer,
                   minute_bar_writer,
                   daily_bar_writer,
                   adjustment_writer,
                   calendar,
                   start_session,
                   end_session,
                   cache,
                   show_progress,
                   output_dir):

    tdx_reader = TdxReader(TDX_DIR)

    symbol_df = get_symbol_list()
    # 只保留未停牌的
    symbol_df = symbol_df[symbol_df['status'] == False]

    # 由于meta,split,dividend 和 行情数据源不同,所以有可能会不同,所以我们这里统一根据

    symbol_map = symbol_df.simplesymbol

    # 更新日期信息
    def update_start_and_end_date(s):
        start_date = start_session.replace(tzinfo=None)
        end_date = end_session.replace(tzinfo=None)
        if s.start_date < start_date:
            s.start_date = start_date
        if s.end_date == pd.Timestamp('1900-01-01') or s.end_date is pd.NaT:
            s.end_date = end_date
        return s
    symbol_df = symbol_df.apply(func=update_start_and_end_date, axis=1)


    # 写入基础信息
    asset_db_writer.write(symbol_df)
    # 写入数据文件
    daily_bar_writer.write(get_hist_data(symbol_df, symbol_map, tdx_reader, start_session, end_session, calendar),
                           show_progress=show_progress)
    # split and diviends
    splits, dividends = zipline_splits_and_dividends(symbol_map)

    # hack for tdx data , for tdx source for shenzhen market, we can not get data before 1991-12-23
    splits_df = pd.concat(splits, ignore_index=True)
    dividends_df = pd.concat(dividends, ignore_index=True)

    splits_df= splits_df.loc[splits_df['effective_date'] > start_session]
    dividends_df = dividends_df.loc[dividends_df['ex_date'] > start_session]
    adjustment_writer.write(
        splits=splits_df,
        dividends=dividends_df,
    )

def get_hist_data(symbol_df, symbol_map, tdx_reader, start_session, end_session, calendar):
    for sid, index in symbol_map.iteritems():
        exchagne = ''
        if symbol_df.loc[sid]['exchange'] == 'SZSE':
            exchagne = 'sz'
        elif symbol_df.loc[sid]['exchange'] == 'SSE':
            exchagne = 'sh'

        try:
            history = tdx_reader.get_df(index, exchagne)

            #print('max-min for %s is %s : %s', (index, history.index[0], history.index[-1]))
            # 去除没有报价信息的内容
            if history.index[0] > pd.Timestamp((end_session.date())):
                continue
        except TdxFileNotFoundException as e:
            #print('symbol %s file no found, ignore it ' % index)
            continue
        # history.to_pickle('/tmp/debug.pickle')

        #reindex
        sessions = calendar.sessions_in_range(start_session, end_session)

        history = history.reindex(
            sessions.tz_localize(None),
            copy=False,
        ).fillna(0.0)

        yield sid, history.sort_index()
    pass

if __name__ == '__main__':

    import tushare as ts
    import pandas as pd

    ts_symbols = ts.get_stock_basics()


    symbols = []


    # 获取股票数据
    i = 0
    total = len(ts_symbols)
    for index, row in ts_symbols.iterrows():
        i = i +1
        if i > 10:
            break

        srow = {}
        srow['t'] = 1
        srow['symbol'] = index
        srow['asset_name'] = row['name']
        symbols.append(srow)

    df_symbols = pd.DataFrame(data=symbols).sort_values('symbol')
    symbol_map = pd.DataFrame.copy(df_symbols.symbol)

    raw_splits, raw_dividends = load_splits_and_dividends()

    splits = []
    dividends = []
    for sid, code in symbol_map.iteritems():
        if code in raw_splits:
            split = pd.DataFrame(data=raw_splits[code])
            split['sid'] = sid
            split.index = split['effective_date'] = pd.DatetimeIndex(split['effective_date'])
            splits.append(split)
        if code in raw_dividends:
            dividend = pd.DataFrame(data = raw_dividends[code])
            dividend['sid'] = sid
            dividend['record_date'] = dividend['declared_date'] = dividend['pay_date'] = pd.NaT
            dividend.index = dividend['ex_date'] = pd.DatetimeIndex(dividend['ex_date'])
            dividends.append(dividend)

    print(pd.concat(splits, ignore_index=True))
    print(pd.concat(dividends, ignore_index=True))



from zipline.data.bundles import register, yahoo_equities
import requests
import os
from pandas_datareader.data import DataReader

"""
For ingest chinese history day bar from Yahoo

From both Shenzhen And Shanghai Stock Exchange
"""

from .all_stocks import get_all_stocks, get_cache_dir


def get_all_yahoo_stock_names(cache=True):

    if cache==False:
        print('Get All Stock List.....')
    all_stocks = get_all_stocks(cache=cache)

    return [full_code(code) for code in all_stocks.index]

def full_code(code):
    if int(code[0]) >= 6:
        return "%s.SS" % code
    else:
        return '%s.SZ' % code

def register_cn_bundle_from_yahoo(name, cache=True):
    """
    register a new bundle of stocks from chinese market from yahoo
    :param name: the name of bundle
    :return: register result
    """
    symbol_list = get_filtered_symbols(cache)

    return register(
        name,
        yahoo_equities(dict(list(zip(symbol_list, symbol_list)))),
    )

def check_code(code):
    """
    XXX: used check_data_reader instead
    :param code:
    :return:
    """
    checkurl = r'http://finance.yahoo.com/_finance_doubledown/api/resource/searchassist;gossipConfig=%7B%22url%22%3A%7B%22host%22%3A%22s.yimg.com%22%2C%22path%22%3A%22%2Fxb%2Fv6%2Ffinance%2Fautocomplete%22%2C%22query%22%3A%7B%22appid%22%3A%22yahoo.com%22%2C%22nresults%22%3A10%2C%22output%22%3A%22yjsonp%22%2C%22region%22%3A%22US%22%2C%22lang%22%3A%22en-US%22%7D%2C%22protocol%22%3A%22https%22%7D%2C%22isJSONP%22%3Atrue%2C%22queryKey%22%3A%22query%22%2C%22resultAccessor%22%3A%22ResultSet.Result%22%2C%22suggestionTitleAccessor%22%3A%22symbol%22%2C%22suggestionMeta%22%3A%5B%22symbol%22%2C%22name%22%2C%22exch%22%2C%22type%22%2C%22exchDisp%22%2C%22typeDisp%22%5D%7D;searchTerm={{CODE}}?bkt=3E0%2507canary&dev_info=0&device=desktop&intl=us&lang=en-US&partner=none&region=US&site=finance&tz=America%2FLos_Angeles&ver=0.101.302&returnMeta=true'
    referer = 'http://finance.yahoo.com/'

    url = checkurl.replace('{{CODE}}', code)

    response = requests.get(url, headers={
        'Referer'       : 'http://finance.yahoo.com/',
        'User-Agent'    : 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/51.0.2704.103 Safari/537.36',
    })
    try:
        data = response.json()
        items = data['data']['items']

        if len(items) > 0:
            for item in items:
                if item['symbol'].upper() == code.upper():
                    print('.')
                    return True

    except Exception as e:
        print(str(e))
        return False

    print('x')
    return False

def check_data_reader(code):
    try:
        DataReader(code, 'yahoo')
    except:
        print('%s ok!' % code)
        return False
    print('%s not ok!' % code)
    return True

def get_filtered_symbols(cache=True):
    cache_dir = get_cache_dir()
    file_path = os.path.join(cache_dir, 'symbols.txt')

    if cache and os.path.isfile(file_path):
        with open(file_path, 'r') as f:
            content = f.read()
            if content:
                return content.split("\n")

    symbols = get_all_yahoo_stock_names(cache)

    if cache==False:
        print('Check availablity from Yahoo...')
    filtered_symbols = list(filter(check_data_reader, symbols))
    print('cache output to %s' % file_path)
    with open(file_path, 'w') as f:
        f.write("\n".join(filtered_symbols))
    print('done!')
    return filtered_symbols

def zipline_cn_databundle_update():
    print('Start to fetch data and update cache')
    get_filtered_symbols(cache=False)


from .yahoo import (
    register_cn_bundle_from_yahoo,
    get_all_yahoo_stock_names,
    check_code,
    get_filtered_symbols,
    zipline_cn_databundle_update,
)

from .all_stocks import (
        get_all_stocks,
)


import matplotlib.pyplot as plt
from zipline.data.bundles import register
from zipline_cn_databundle.squant_source import squant_bundle
import pandas as pd
import os

from zipline.api import (
    schedule_function,
    symbol,
    order_target_percent,
    date_rules,
    record
)
import re
from zipline.algorithm import TradingAlgorithm
from zipline.finance.trading import TradingEnvironment
from zipline.utils.calendars import get_calendar, register_calendar
from zipline.finance import trading
from zipline.utils.factory import create_simulation_parameters
from zipline.data.bundles.core import load
from zipline.data.data_portal import DataPortal

from zipline_cn_databundle.loader import load_market_data

# register SHSZ

from cn_stock_holidays.zipline.default_calendar import shsz_calendar


bundle = 'cn_squant'

start_session_str = '2011-01-05'

register(
        bundle,
        squant_bundle,
        "SHSZ",
        pd.Timestamp(start_session_str, tz='utc'),
        pd.Timestamp('2016-10-31', tz='utc')
        )


bundle_data = load(
    bundle,
    os.environ,
    None,
)

prefix, connstr = re.split(
    r'sqlite:///',
    str(bundle_data.asset_finder.engine.url),
    maxsplit=1,
)

env = trading.environment = TradingEnvironment(asset_db_path=connstr,
                                               trading_calendar=shsz_calendar,
                                               bm_symbol='000001.SS',
                                               load=load_market_data)


first_trading_day = \
    bundle_data.equity_minute_bar_reader.first_trading_day
data = DataPortal(
    env.asset_finder, shsz_calendar,
    first_trading_day=first_trading_day,
    equity_minute_reader=bundle_data.equity_minute_bar_reader,
    equity_daily_reader=bundle_data.equity_daily_bar_reader,
    adjustment_reader=bundle_data.adjustment_reader,
)


strategy = """
from zipline.api import (
    schedule_function,
    symbol,
    order_target_percent,
    date_rules,
    record
)

def initialize(context):
    schedule_function(handle_daily_data, date_rules.every_day())

def handle_daily_data(context, data):
    sym = symbol('000001.SZ')

    # 计算均线
    short_mavg = data.history(sym, 'close', 5, '1d').mean()
    long_mavg = data.history(sym, 'close', 10, '1d').mean()

    # 交易逻辑
    if short_mavg > long_mavg:
        # 满仓
        order_target_percent(sym, 1)
    elif short_mavg < long_mavg:
        # 清仓
        order_target_percent(sym, 0)

    # Save values for later inspection
    record(价格=data.current(sym, 'price'),
           short_mavg=short_mavg,
           long_mavg=long_mavg)
"""

def analyze(context, perf):
    fig = plt.figure(figsize=(14,13))
    ax1 = fig.add_subplot(211)
    perf.portfolio_value.plot(ax=ax1, grid=True)
    ax1.set_ylabel('portfolio value in $')

    ax2 = fig.add_subplot(212)
    perf['GOOG'].plot(ax=ax2)
    perf[['short_mavg', 'long_mavg']].plot(ax=ax2)

    perf_trans = perf.ix[[t != [] for t in perf.transactions]]
    buys = perf_trans.ix[[t[0]['amount'] > 0 for t in perf_trans.transactions]]
    sells = perf_trans.ix[
        [t[0]['amount'] < 0 for t in perf_trans.transactions]]
    ax2.plot(buys.index, perf.short_mavg.ix[buys.index],
             '^', markersize=4, color='m')
    ax2.plot(sells.index, perf.short_mavg.ix[sells.index],
             'v', markersize=4, color='k')
    ax2.set_ylabel('price in $')
    plt.legend(loc=0)
    plt.show()
    plt.imsave("output.png")


if __name__ == '__main__':
    sim_params = create_simulation_parameters(
        start=pd.to_datetime(start_session_str + " 00:00:00").tz_localize("Asia/Shanghai"),
        end=pd.to_datetime("2012-01-01 00:00:00").tz_localize("Asia/Shanghai"),
        data_frequency="daily", emission_rate="daily", trading_calendar=shsz_calendar)

    algor_obj = TradingAlgorithm(script=strategy,
                                 sim_params=sim_params,
                                 env=trading.environment,
                                 trading_calendar=shsz_calendar)
    # not use run method of TradingAlgorithm
    #perf_manual = algor_obj.run(data)
    #perf_manual.to_pickle('/tmp/perf.pickle')

    algor_obj.data_portal = data
    algor_obj._assets_from_source = \
            algor_obj.trading_environment.asset_finder.retrieve_all(
                    algor_obj.trading_environment.asset_finder.sids
                    )
    algor_obj.perf_tracker = None
    import demjson
    import pickle
    import pprint
    try:
        perfs = []
        for perf in algor_obj.get_generator():
                perfs.append(perf)
                print('-' * 40)
                try:
                    print(demjson.encode(perf))
                except Exception as e:
                    pprint.pprint(perf)
                print('-' * 40)

        pickle.dump(perfs, open('/tmp/raw_perfs.pickle', 'wb'))
        daily_stats = algor_obj._create_daily_stats(perfs)
        print('*' * 40)
        print('daily_stats')
        print('*' * 40)
        print(daily_stats)
        daily_stats.to_pickle('/tmp/perf.pickle')
    finally:
        algor_obj.data_portal = None



from setuptools import setup, find_packages

setup(
    name = "zipline-cn-databundle",
    version = "0.5",
    author= 'RainX',
    description='ingest zipline databundle source for chinese market',
    packages = find_packages(),
    install_requires=[
        'pandas-datareader',
    ],
    entry_points={
        'console_scripts': [
            'zipline-cn-databundle-update=zipline_cn_databundle:zipline_cn_databundle_update',
            'gen-all-index-benchmark-data=zipline_cn_databundle.index_list.__init__:gen_data',
        ]
    }
)

